using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Activities;
using System.IO;
using System.Diagnostics;

namespace Benday.TeamBuild2012.Activities
{
    public abstract class CommandActivity : BuildActivityBase
    {
        public CommandActivity()
        {

        }

        public void RunCommand(CodeActivityContext context, string filename, string args, string operationName)
        {
            LogDebugVariableValue(context, "operationName", operationName);
            LogDebugVariableValue(context, "filename", filename);
            LogDebugVariableValue(context, "args", args);

            ProcessStartInfo info = null;
            Process process = null;

            info = new ProcessStartInfo(filename, args);

            info.CreateNoWindow = true;
            info.UseShellExecute = false;
            info.RedirectStandardError = true;
            info.RedirectStandardOutput = true;

            process = Process.Start(info);

            string standardErrorText = process.StandardError.ReadToEnd();
            string standardOutputText = process.StandardOutput.ReadToEnd();

            process.WaitForExit();

            if (process.ExitCode == 0 &&
                String.IsNullOrWhiteSpace(standardErrorText) == true)
            {
                string message = String.Format("{0} succeeded: {1})",
                                     operationName,
                                     SafeTrim(standardOutputText));

                LogDebugMessage(context, message);
            }
            else if (process.ExitCode != 0 ||
                String.IsNullOrWhiteSpace(standardErrorText) == false)
            {
                string message = String.Format("{0} command failed with an error of '{1}'.)",
                                        operationName,
                                        SafeTrim(standardErrorText));

                throw new InvalidOperationException(message);
            }
        }
    }
}
