using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Activities;
using System;
using System.Activities;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;

namespace Benday.TeamBuild2012.Activities
{

    [BuildActivity(HostEnvironmentOption.Agent)]
    public sealed class DeployDatabase : CommandActivity
    {
        public override string ActivityName
        {
            get { return "DeployDatabase"; }
        }

        [RequiredArgument]
        public InArgument<string> DacpacPath { get; set; }

        [RequiredArgument]
        public InArgument<string> ProfilePath { get; set; }

        private string m_PathToSqlPackageExe;

        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                LogDebugMessage(context, "Initializing...");

                string tempDacpacPath = DacpacPath.Get(context);
                string tempProfilePath = ProfilePath.Get(context);

                LogDebugVariableValue(context, "DacpacPath", tempDacpacPath);
                LogDebugVariableValue(context, "ProfilePath", tempProfilePath);

                LocateSqlPackageExeAndAssert(context);

                RunDeploy(context, tempDacpacPath, tempProfilePath);

                LogUsefulMessage(context, "Database deployment completed.");
            }
            catch (Exception ex)
            {
                LogException(context, ex);
            }
        }

        private void LocateSqlPackageExeAndAssert(CodeActivityContext context)
        {
            string location1 = @"C:\Program Files\Microsoft SQL Server\110\DAC\bin\SqlPackage.exe";
            string location2 = @"C:\Program Files (x86)\Microsoft SQL Server\110\DAC\bin\SqlPackage.exe";

            if (File.Exists(location1) == true)
            {
                m_PathToSqlPackageExe = location1;
            }
            else if (File.Exists(location2) == true)
            {
                m_PathToSqlPackageExe = location2;
            }

            if (String.IsNullOrWhiteSpace(m_PathToSqlPackageExe) == true ||
                File.Exists(m_PathToSqlPackageExe) == false)
            {
                string message = String.Format(
                                    "Could not locate SqlPackage.exe at either '{0}' or '{1}'.",
                                    location1, location2);
                
                throw new InvalidOperationException(message);
            }
            else
            {
                LogDebugVariableValue(context, "Path to SqlPackage.exe", m_PathToSqlPackageExe);
            }
        }

        private void RunDeploy(CodeActivityContext context, string dacpacPath, string profilePath)
        {
            // SqlPackage /Action:Publish /SourceFile:"Benday.DatabaseDemo.PersonDatabase.dacpac" /Profile:"Production.publish.xml"

            /*
            string argsTemplate = "/C \"{0}\" /Action:Publish /SourceFile:\"{1}\" /Profile:\"{2}\"";

            string args = String.Format(argsTemplate, m_PathToSqlPackageExe, dacpacPath, profilePath);

            string command = "cmd.exe";
            */

            string argsTemplate = "/Action:Publish /SourceFile:\"{1}\" /Profile:\"{2}\"";

            string args = String.Format(argsTemplate, m_PathToSqlPackageExe, dacpacPath, profilePath);

            string command = m_PathToSqlPackageExe;

            RunCommand(context, command, args, "SqlPackage.exe");
        }
    }
}