﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Activities;
using Microsoft.TeamFoundation.Build.Client;
using Microsoft.TeamFoundation.Build.Workflow.Tracking;
using Microsoft.TeamFoundation.Build.Workflow.Activities;

namespace Benday.TeamBuild2012.Activities
{
    [BuildActivity(HostEnvironmentOption.Agent)]
    public sealed class IisDeleteWebApplication : BuildActivityBase
    {
        public override string ActivityName
        {
            get { return "IisDeleteWebApplication"; }
        }

        private IisUtility m_iisUtilityInstance;
        public IisUtility IisUtilityInstance
        {
            get
            {
                if (m_iisUtilityInstance == null)
                {
                    m_iisUtilityInstance = new IisUtility();
                }

                return m_iisUtilityInstance;
            }
        }

        [RequiredArgument]
        public InArgument<string> SiteName { get; set; }

        [RequiredArgument]
        public InArgument<string> ApplicationName { get; set; }

        private string VerifySiteName(string siteName)
        {
            if (String.IsNullOrEmpty(siteName) == true)
            {
                var sites = IisUtilityInstance.GetSites();

                if (sites == null || sites.Count < 1)
                {
                    throw new InvalidOperationException("Could not determine default site name because IIS does not contain any sites.");
                }
                else
                {
                    return sites[0];
                }
            }
            else
            {
                return siteName;
            }
        }

        protected override void Execute(CodeActivityContext context)
        {
            try
            {
                LogDebugMessage(context, "Initializing...");

                string tempSiteName = SiteName.Get(context);
                string tempApplicationName = ApplicationName.Get(context);

                LogDebugVariableValue(context, "SiteName", tempSiteName);
                LogDebugVariableValue(context, "ApplicationName", tempApplicationName);
               
                tempSiteName = VerifySiteName(tempSiteName);

                LogDebugMessage(context, "Calling DeleteApplication()...");

                IisUtilityInstance.DeleteApplication(tempSiteName, tempApplicationName);

                LogUtility.WriteToLog(context, String.Format("IisDeleteWebApplication: Deleted web application.  (Site: {0}, Application: {1})",
                                                   tempSiteName,
                                                   tempApplicationName), BuildMessageImportance.High);

                LogUsefulMessage(context, String.Format("Deleted web application '{0}' from '{1}'.", tempApplicationName, tempSiteName));
            }
            catch(Exception ex)
            {
                LogException(context, ex);

                throw;
            }
        }   
    }
}
