﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Web.Administration;

namespace Benday.TeamBuild2012.Activities
{
    public class IisUtility
    {
        private ServerManager m_mgr;

        public IisUtility()
        {
            m_mgr = new ServerManager();
        }

        public IList<string> GetSites()
        {
            List<string> returnValue = new List<string>();

            foreach (var site in m_mgr.Sites)
            {
                returnValue.Add(site.Name);
            }

            return returnValue;
        }

        public bool ApplicationExists(string siteName, string applicationName)
        {
            if (applicationName == null)
            {
                throw new ArgumentNullException("applicationName", "Argument cannot be null.");
            }

            if (SiteExists(siteName) == false)
            {
                return false;
            }
            else
            {
                var apps = GetApplications(siteName);

                if (applicationName.StartsWith("/") == false)
                {
                    applicationName = "/" + applicationName;
                }

                var match = (from temp in apps
                             where temp == applicationName
                             select temp).FirstOrDefault();

                if (match == null)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }            
        }

        public bool SiteExists(string siteName)
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName", "Argument cannot be null.");
            }

            var site = m_mgr.Sites[siteName];

            if (site == null)
            {
                return false;
            }
            else
            {
                return true;
            }
        }

        private void AssertSiteExists(string siteName)
        {
            if (SiteExists(siteName) == false)
            {
                throw new InvalidOperationException(String.Format("Unknown site name ('{0}').", siteName));
            }
        }

        public IList<string> GetApplications(string siteName)
        {
            AssertSiteExists(siteName);
        
            List<string> returnValue = new List<string>();

            foreach (var app in m_mgr.Sites[siteName].Applications)
            {
                returnValue.Add(app.Path);
            }

            return returnValue;
        }

        public void DeleteApplication(string siteName, string applicationName)
        {
            if (ApplicationExists(siteName, applicationName) == true)
            {
                if (applicationName.StartsWith("/") == false)
                {
                    applicationName = "/" + applicationName;
                }

                var app = m_mgr.Sites[siteName].Applications[applicationName];

                m_mgr.Sites[siteName].Applications.Remove(app);

                m_mgr.CommitChanges();
            }
        }

        public void CreateApplication(string siteName, string applicationName, string path)
        {
            if (applicationName == null)
            {
                throw new ArgumentNullException("applicationName", "Argument cannot be null.");
            }

            if (path == null)
            {
                throw new ArgumentNullException("path", "Argument cannot be null.");
            }

            AssertSiteExists(siteName);

            if (ApplicationExists(siteName, applicationName) == true)
            {
                throw new InvalidOperationException(
                    String.Format("There is already an application with name '{0}'.", applicationName));
            }

            if (applicationName.StartsWith("/") == false)
            {
                applicationName = "/" + applicationName;
            }

            m_mgr.Sites[siteName].Applications.Add(applicationName, path);

            m_mgr.CommitChanges();            
        }

        public IList<string> GetAppPools()
        {
            List<string> returnValue = new List<string>();

            foreach (var pool in m_mgr.ApplicationPools)
            {
                returnValue.Add(pool.Name);
            }

            return returnValue;
        }

        public bool AppPoolExists(string appPoolName)
        {
            if (appPoolName == null)
            {
                throw new ArgumentNullException("appPoolName", "Argument cannot be null.");
            }

            var pools = GetAppPools();

            if (pools == null)
            {
                return false;
            }
            else
            {
                var match = (from temp in pools
                             where temp == appPoolName
                             select temp).FirstOrDefault();

                if (match == null)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            
        }

        private void AssertApplicationExists(string siteName, string applicationName)
        {
            if (ApplicationExists(siteName, applicationName) == false)
            {
                throw new InvalidOperationException("Application does not exist.");
            }
        }

        public string GetAppPoolForApplication(string siteName, string applicationName)
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName", "Argument cannot be null.");
            }

            if (applicationName == null)
            {
                throw new ArgumentNullException("applicationName", "Argument cannot be null.");
            }

            if (applicationName.StartsWith("/") == false)
            {
                applicationName = "/" + applicationName;
            }

            AssertApplicationExists(siteName, applicationName);

            var app = m_mgr.Sites[siteName].Applications[applicationName];

            return app.ApplicationPoolName;
        }

        private void AssertAppPoolExists(string appPoolName)
        {
            if (AppPoolExists(appPoolName) == false)
            {
                throw new InvalidOperationException(String.Format("Application pool '{0}' does not exist.", appPoolName));
            }
        }

        public void SetAppPoolForApplication(string siteName, string applicationName, string appPoolName)
        {
            if (String.IsNullOrEmpty(siteName))
                throw new ArgumentException("siteName is null or empty.", "siteName");
            if (String.IsNullOrEmpty(applicationName))
                throw new ArgumentException("applicationName is null or empty.", "applicationName");
            if (String.IsNullOrEmpty(appPoolName))
                throw new ArgumentException("appPoolName is null or empty.", "appPoolName");

            if (applicationName.StartsWith("/") == false)
            {
                applicationName = "/" + applicationName;
            }

            AssertAppPoolExists(appPoolName);
            AssertApplicationExists(siteName, applicationName);

            var app = m_mgr.Sites[siteName].Applications[applicationName].ApplicationPoolName = appPoolName;
            m_mgr.CommitChanges();
        }

        private void AssertElementNotNull(object element, string name)
        {
            if (element == null)
            {
                throw new InvalidOperationException(String.Format("ConfigurationElement named '{0}' was null.", name));
            }
        }
        
        public SecuritySettings GetSecurityForApplication(string siteName, string applicationName)
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName", "Argument cannot be null.");
            }

            if (applicationName == null)
            {
                throw new ArgumentNullException("applicationName", "Argument cannot be null.");
            }

            if (applicationName.StartsWith("/") == false)
            {
                applicationName = "/" + applicationName;
            }

            AssertApplicationExists(siteName, applicationName);

            Configuration appHostConfig = m_mgr.GetApplicationHostConfiguration();

            string locationPath = String.Format("{0}{1}", siteName, applicationName);

            var anonymousAuthentication = appHostConfig.GetSection("system.webServer/security/authentication/anonymousAuthentication", locationPath);
            AssertElementNotNull(anonymousAuthentication, "anonymousAuthentication");

            var basicAuthentication = appHostConfig.GetSection("system.webServer/security/authentication/basicAuthentication", locationPath);
            AssertElementNotNull(basicAuthentication, "basicAuthentication");

            var windowsAuthentication = appHostConfig.GetSection("system.webServer/security/authentication/windowsAuthentication", locationPath);
            AssertElementNotNull(windowsAuthentication, "windowsAuthentication");
            
            SecuritySettings settings = new SecuritySettings();

            AssertElementNotNull(anonymousAuthentication["enabled"], "anonymousAuthentication.enabled");
            AssertElementNotNull(windowsAuthentication["enabled"], "windowsAuthentication.enabled");
            AssertElementNotNull(basicAuthentication["enabled"], "basicAuthentication.enabled");
            
            settings.Anonymous = Boolean.Parse(anonymousAuthentication["enabled"].ToString());
            settings.Windows = Boolean.Parse(windowsAuthentication["enabled"].ToString());
            settings.Basic = Boolean.Parse(basicAuthentication["enabled"].ToString());

            return settings;            
        }

        public void SetSecurityForApplication(string siteName, string applicationName, SecuritySettings settings)
        {
            if (siteName == null)
            {
                throw new ArgumentNullException("siteName", "Argument cannot be null.");
            }

            if (applicationName == null)
            {
                throw new ArgumentNullException("applicationName", "Argument cannot be null.");
            }

            if (applicationName.StartsWith("/") == false)
            {
                applicationName = "/" + applicationName;
            }

            AssertApplicationExists(siteName, applicationName);
            
            Configuration appHostConfig = m_mgr.GetApplicationHostConfiguration();

            string locationPath = String.Format("{0}{1}", siteName, applicationName);

            var anonymousAuthentication = appHostConfig.GetSection("system.webServer/security/authentication/anonymousAuthentication", locationPath);
            AssertElementNotNull(anonymousAuthentication, "anonymousAuthentication");

            var basicAuthentication = appHostConfig.GetSection("system.webServer/security/authentication/basicAuthentication", locationPath);
            AssertElementNotNull(basicAuthentication, "basicAuthentication");

            var windowsAuthentication = appHostConfig.GetSection("system.webServer/security/authentication/windowsAuthentication", locationPath);
            AssertElementNotNull(windowsAuthentication, "windowsAuthentication");

            anonymousAuthentication["enabled"] = settings.Anonymous.ToString().ToLower();
            basicAuthentication["enabled"] = settings.Basic.ToString().ToLower();
            windowsAuthentication["enabled"] = settings.Windows.ToString().ToLower();

            m_mgr.CommitChanges();
        }
    }
}
