using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.TeamBuild2012;
using System.IO;
using Benday.TeamBuild2012.Activities;

namespace Benday.TeamBuild2012.UnitTests
{
    [TestClass]
    public class DatabaseConfigurationFileEditorFixture : UnitTestBase
    {
        public DatabaseConfigurationFileEditorFixture()
        {
        }

        private string GetPathToTestWebApp()
        {
            string path = TestContext.TestDeploymentDir;

            Assert.IsTrue(Directory.Exists(path), "Path does not exist.  '{0}'", path);

            return path;
        }

        private void AssertConnectionString(string expected, string connectionName)
        {
            string filename = Path.Combine(GetPathToTestWebApp(), "web.config");

            Assert.IsTrue(File.Exists(filename), "Could not find config file at '{0}'.", filename);
            
            string actual = DatabaseConfigurationFileEditor.GetConnectionString(filename, connectionName);

            Assert.AreEqual<string>(expected, actual, "Connection string '{0}' didn't match.", connectionName);
        }

        private void UpdateConnectionString(string updatedValue, string connectionName)
        {
            string filename = Path.Combine(GetPathToTestWebApp(), "web.config");

            Assert.IsTrue(File.Exists(filename), "Could not find config file at '{0}'.", filename);

            DatabaseConfigurationFileEditor.SetConnectionString(filename, connectionName, updatedValue);
        }

        [TestMethod]
        public void GetConnectionString()
        {
            AssertConnectionString(
                @"data source=.\SQLEXPRESS;Integrated Security=SSPI;AttachDBFilename=|DataDirectory|\aspnetdb.mdf;User Instance=true", 
                "ApplicationServices");

            AssertConnectionString(
                @"data source=.\SQLEXPRESS;Integrated Security=SSPI;",
                "Connection2");
        }

        [TestMethod]
        public void SetConnectionStringWhenNoConnectionStringExists()
        {
            string updatedValue = @"updatedValue";

            string connectionStringName = Guid.NewGuid().ToString();

            UpdateConnectionString(updatedValue, connectionStringName);

            AssertConnectionString(
                updatedValue,
                connectionStringName);
        }

        [TestMethod]
        public void SetConnectionStringWhenConnectionStringExists()
        {
            string updatedValue = @"updatedValue";

            string connectionStringName = "Connection2";

            UpdateConnectionString(updatedValue, connectionStringName);

            AssertConnectionString(
                updatedValue,
                connectionStringName);
        }


    }
}
