﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.TeamBuild2012;
using System.Xml.Linq;
using Benday.TeamBuild2012.Activities;

namespace Benday.TeamBuild2012.UnitTests
{
    public class UnitTestBase
    {
        protected const string STR_DefaultWebSite = "Default Web Site";
        protected const string STR_TestWebApplication = "TestWebApplication";

        public TestContext TestContext { get; set; }

        protected virtual void OnTestInitialize()
        {

        }

        [TestInitialize]
        public void TestInitialize()
        {
            OnTestInitialize();
        }

        protected void DeleteApplicationIfItExists(string site, string applicationName)
        {
            if (UtilityInstance.ApplicationExists(site, applicationName) == true)
            {
                UtilityInstance.DeleteApplication(site, applicationName);

                Assert.IsFalse(UtilityInstance.ApplicationExists(site, applicationName), "Application should be gone.");
            }
        }

        private IisUtility m_utilityInstance;
        public IisUtility UtilityInstance
        {
            get
            {
                if (m_utilityInstance == null)
                    m_utilityInstance = new IisUtility();
                return m_utilityInstance;
            }
        }

        protected void AssertAttributeValue(XElement element, string attributeName, string expectedAttributeValue)
        {
            Assert.IsNotNull(element, "Element was null.");
            Assert.IsTrue(element.HasAttributes, "Element should have attributes.");
            var attribute = element.Attribute(attributeName);
            Assert.IsNotNull(attributeName, "Element did not contain an attribute named '{0}'.", attributeName);
            Assert.AreEqual<string>(expectedAttributeValue, attribute.Value, "Attribute value for '{0}' was wrong.", attributeName);
        }
        
    }
}